#include "bench.h"

/*
 *	There _must_ be something better than rusage.ru_utime with a higher
 *	resolution ...
 *
 *	Linux isn't using rusage.ru_nvcsw and rusage.ru_nivcsw, which might
 *	have the highest resolution we could get for this ...
 */

#include <stdio.h>
#include <sys/time.h>
#include <sys/resource.h>

#define TIMER_DEBUG 0

long timer_buffer[CONTEXT_NR];
long timer_value[CONTEXT_NR];

long timer_current() {
	struct rusage u;
	getrusage(RUSAGE_SELF, &u);

#if TIMER_DEBUG
	fprintf(stderr, "\nTIMER: utime = (%ld, %ld) -> (%ld)",
	        u.ru_utime.tv_sec, u.ru_utime.tv_usec,
		(u.ru_utime.tv_sec * 100) +
		(u.ru_utime.tv_usec / 10000));
#endif

	return	(u.ru_utime.tv_sec * 100) +
		(u.ru_utime.tv_usec / 10000);
}

void timer_start(int context) {
	timer_buffer[context] = timer_current();
}

void timer_stop(int context) {
	timer_value[context] = timer_current() - timer_buffer[context];
}

long timer_reset(int context) {
#if TIMER_DEBUG
	fprintf(stderr, "\nTIMER: result = %-32ld", timer_value[context]);
#endif
	timer_buffer[context] = timer_value[context];
	timer_value[context] = 0;
	return timer_buffer[context];
}

